"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const Logger_1 = __importDefault(require("./Logger"));
const stream_1 = require("stream");
const public_ip_1 = __importDefault(require("public-ip"));
class S3 {
    constructor(s3Url, s3Params) {
        this.url = s3Url;
        this.params = s3Params;
        this.bucketName = s3Params.bucketName;
    }
    initialise() {
        return __awaiter(this, void 0, void 0, function* () {
            const s3UrlBase = new aws_sdk_1.default.Endpoint(this.url);
            this.s3Handler = new aws_sdk_1.default.S3({
                endpoint: s3UrlBase,
                accessKeyId: this.params.keyId,
                secretAccessKey: this.params.secretAccessKey,
                s3ForcePathStyle: s3UrlBase.protocol === 'http:',
            });
            try {
                if ((yield this.bucketExists(this.bucketName)) === true) {
                    return true;
                }
            }
            catch (err) {
                throw new Error(`Unable to connect to S3, either S3 login credentials are wrong or bucket cannot be found
                            Bucket used: ${this.bucketName}
                            End point used: ${s3UrlBase.href}
                            Public IP used: ${yield public_ip_1.default.v4()}`);
            }
        });
    }
    bucketExists(bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.s3Handler.headBucket({ Bucket: bucket }, function (err) {
                    err ? reject(err) : resolve(true);
                });
            });
        });
    }
    uploadBlob(key, data, eTag, version) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                Bucket: this.bucketName,
                Key: key,
                Metadata: { etag: eTag, version },
                Body: this.bufferToStream(data),
            };
            try {
                this.s3Handler.upload(params, function (err, data) {
                    if (err) {
                        Logger_1.default.log(`Not able to upload ${key}: ${err}`);
                    }
                });
            }
            catch (err) {
                Logger_1.default.log('S3 error', err);
            }
        });
    }
    downloadBlob(key, version = '1') {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.s3Handler.getObject({ Bucket: this.bucketName, Key: key }, (err, val) => __awaiter(this, void 0, void 0, function* () {
                    if (val) {
                        if (val.Metadata.version !== version) {
                            val.Metadata.etag = undefined;
                        }
                        resolve(val);
                    }
                    else if (err && err.statusCode === 404) {
                        resolve();
                    }
                    else {
                        reject(err);
                    }
                }));
            }).catch((err) => {
                return err;
            });
        });
    }
    // Only for testing purpose
    deleteBlob(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.s3Handler.deleteObject(key, (err, val) => {
                    err ? reject(err) : resolve(val);
                });
            });
        });
    }
    bufferToStream(binary) {
        return new stream_1.Readable({
            read() {
                this.push(binary);
                this.push(null);
            },
        });
    }
}
exports.default = S3;
//# sourceMappingURL=S3.js.map