"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisKvs = void 0;
const os_1 = require("os");
const p_map_1 = __importDefault(require("p-map"));
class RedisKvs {
    constructor(redisClient, dbName, keyMapping) {
        this.redisClient = redisClient;
        this.dbName = dbName;
        this.keyMapping = keyMapping;
        if (keyMapping) {
            this.invertedKeyMapping = Object.entries(keyMapping).reduce((acc, [key, val]) => (Object.assign(Object.assign({}, acc), { [val]: key })), {});
        }
    }
    get(prop) {
        return new Promise((resolve, reject) => {
            this.redisClient.hget(this.dbName, prop, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    const d = JSON.parse(val);
                    const mappedVal = this.mapKeysGet(d);
                    resolve(mappedVal);
                }
            });
        });
    }
    getMany(prop) {
        return new Promise((resolve, reject) => {
            this.redisClient.hmget(this.dbName, prop, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val
                        .map((a) => JSON.parse(a))
                        .reduce((acc, val, index) => {
                        return Object.assign(Object.assign({}, acc), { [prop[index]]: this.mapKeysGet(val) });
                    }, {}));
                }
            });
        });
    }
    exists(prop) {
        return p_map_1.default(prop, (key) => {
            return new Promise((resolve, reject) => {
                this.redisClient.hexists(this.dbName, key, (err, val) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve({ [key]: val });
                    }
                });
            });
        }, { concurrency: os_1.cpus().length * 4 }).then((vals) => vals.reduce((acc, val) => Object.assign(acc, val), {}));
    }
    set(prop, val) {
        return new Promise((resolve, reject) => {
            const valToSet = this.mapKeysSet(val);
            const normalisedVal = typeof valToSet !== 'string' ? JSON.stringify(valToSet) : valToSet;
            this.redisClient.hset(this.dbName, prop, normalisedVal, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val);
                }
            });
        });
    }
    setMany(val) {
        return new Promise((resolve, reject) => {
            const numKeys = Object.keys(val).length;
            if (!numKeys) {
                resolve();
                return;
            }
            const normalisedVal = Object.entries(val)
                .reduce((acc, [key, val]) => {
                const newVal = this.mapKeysSet(val);
                acc[key] = typeof newVal !== 'string' ? JSON.stringify(newVal) : newVal;
                return acc;
            }, {});
            this.redisClient.hmset(this.dbName, normalisedVal, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val);
                }
            });
        });
    }
    delete(prop) {
        return new Promise((resolve, reject) => {
            this.redisClient.hdel(this.dbName, prop, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val);
                }
            });
        });
    }
    deleteMany(prop) {
        return new Promise((resolve, reject) => {
            this.redisClient.hdel(this.dbName, prop.join(' '), (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val);
                }
            });
        });
    }
    keys() {
        return new Promise((resolve, reject) => {
            this.redisClient.hkeys(this.dbName, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val);
                }
            });
        });
    }
    len() {
        return new Promise((resolve, reject) => {
            this.redisClient.hlen(this.dbName, (err, val) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(val);
                }
            });
        });
    }
    iterateItems(numWorkers, func) {
        return __awaiter(this, void 0, void 0, function* () {
            const workers = Array.from(Array(numWorkers).keys());
            let scanCursor = '0';
            let depleted = false;
            let pendingScan = Promise.resolve(null);
            yield p_map_1.default(workers, (workerId) => __awaiter(this, void 0, void 0, function* () {
                while (true) {
                    pendingScan = pendingScan.then(() => this.scan(scanCursor)
                        .then((result) => {
                        scanCursor = result.cursor;
                        return result;
                    }));
                    const { cursor, items } = yield pendingScan;
                    if (depleted)
                        break;
                    const parsedItems = items.reduce((acc, [key, strVal]) => {
                        return Object.assign(Object.assign({}, acc), { [key]: this.mapKeysGet(JSON.parse(strVal)) });
                    }, {});
                    if (cursor === '0')
                        depleted = true;
                    scanCursor = cursor;
                    yield func(parsedItems, workerId);
                }
            }), { concurrency: numWorkers });
        });
    }
    scan(scanCursor) {
        return new Promise((resolve, reject) => {
            this.redisClient.hscan(this.dbName, scanCursor, (err, [cursor, data]) => {
                if (err)
                    return reject(err);
                const items = Array.from(data, (x, k) => k % 2 ? undefined : [x, data[k + 1]]).filter((x) => x);
                resolve({ cursor, items });
            });
        });
    }
    flush() {
        return new Promise((resolve, reject) => {
            this.redisClient.del(this.dbName, (err) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        });
    }
    mapKeysGet(obj) {
        let mappedVal = obj;
        if (obj && this.keyMapping && typeof obj === 'object') {
            mappedVal = Object.entries(obj)
                .reduce((acc, [key, val]) => {
                if (this.keyMapping[key]) {
                    const newKey = this.keyMapping[key];
                    return Object.assign(Object.assign({}, acc), { [newKey]: val });
                }
                else {
                    return Object.assign(Object.assign({}, acc), { [key]: val });
                }
            }, {});
        }
        return mappedVal;
    }
    mapKeysSet(obj) {
        let mappedVal = obj;
        if (obj && this.invertedKeyMapping && typeof obj === 'object') {
            mappedVal = Object.entries(obj)
                .reduce((acc, [key, val]) => {
                if (this.invertedKeyMapping[key]) {
                    const newKey = this.invertedKeyMapping[key];
                    return Object.assign(Object.assign({}, acc), { [newKey]: val });
                }
                else {
                    return Object.assign(Object.assign({}, acc), { [key]: val });
                }
            }, {});
        }
        return mappedVal;
    }
}
exports.RedisKvs = RedisKvs;
//# sourceMappingURL=RedisKvs.js.map