"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderDesktopArticle = exports.renderArticle = void 0;
const domino_1 = __importDefault(require("domino"));
const Templates_1 = require("../Templates");
const Logger_1 = __importDefault(require("../Logger"));
const stores_1 = require("../stores");
const misc_1 = require("./misc");
const const_1 = require("./const");
exports.renderArticle = (json, articleId, dump, capabilities) => __awaiter(void 0, void 0, void 0, function* () {
    const articleDetail = yield stores_1.articleDetailXId.get(articleId);
    const isMainPage = dump.isMainPage(articleId);
    if (isMainPage || (capabilities.veApiAvailable && !capabilities.desktopRestApiAvailable)) {
        const html = exports.renderDesktopArticle(json, articleId, articleDetail, isMainPage);
        const strippedTitle = misc_1.getStrippedTitleFromHtml(html);
        return [{
                articleId,
                displayTitle: strippedTitle || articleId.replace('_', ' '),
                html,
            }];
    }
    const result = [];
    let html = json;
    // Paginate when there are more than 200 subCategories
    const numberOfPagesToSplitInto = Math.max(Math.ceil((articleDetail.subCategories || []).length / 200), 1);
    for (let i = 0; i < numberOfPagesToSplitInto; i++) {
        const pageId = i === 0 ? '' : `__${i}`;
        const _articleId = articleId + pageId;
        const _articleDetail = Object.assign({}, articleDetail, {
            subCategories: (articleDetail.subCategories || []).slice(i * 200, (i + 1) * 200),
            nextArticleId: numberOfPagesToSplitInto > i + 1 ? `${articleId}__${i + 1}` : null,
            prevArticleId: (i - 1) > 0 ?
                `${articleId}__${i - 1}`
                : (i - 1) === 0
                    ? articleId
                    : null,
        });
        if ((articleDetail.subCategories || []).length > 200) {
            yield stores_1.articleDetailXId.set(_articleId, _articleDetail);
        }
        // We don't really know the nature of 'json' variable because
        // of weak software architecture. Got there is correct json.
        if (json.lead) {
            html = renderMCSArticle(json, dump, _articleId, _articleDetail);
        }
        let strippedTitle = misc_1.getStrippedTitleFromHtml(html);
        if (!strippedTitle) {
            const title = (json.lead || { displaytitle: articleId }).displaytitle;
            const doc = domino_1.default.createDocument(`<span class='mw-title'>${title}</span>`);
            strippedTitle = doc.getElementsByClassName('mw-title')[0].textContent;
        }
        result.push({
            articleId: _articleId,
            displayTitle: (strippedTitle || articleId.replace(/_/g, ' ')) + (i === 0 ? '' : `/${i}`),
            html,
        });
    }
    return result;
});
const injectHeader = (content, articleId, articleDetail) => {
    const doc = domino_1.default.createDocument(content);
    const header = doc.createElement('h1');
    header.appendChild(doc.createTextNode(articleDetail.title));
    header.classList.add('article-header');
    const target = doc.querySelector('body.mw-body-content');
    target.insertAdjacentElement('afterbegin', header);
    return doc.documentElement.outerHTML;
};
exports.renderDesktopArticle = (json, articleId, articleDetail, isMainPage = false) => {
    if (!json) {
        throw new Error(`Cannot render [${json}] into an article`);
    }
    if (json.visualeditor) {
        // Testing if article has been deleted between fetching list and downloading content.
        if (json.visualeditor.oldid === 0) {
            Logger_1.default.error(const_1.DELETED_ARTICLE_ERROR);
            throw new Error(const_1.DELETED_ARTICLE_ERROR);
        }
        return isMainPage ? json.visualeditor.content : injectHeader(json.visualeditor.content, articleId, articleDetail);
    }
    else if (json.contentmodel === 'wikitext' || (json.html && json.html.body)) {
        return json.html.body;
    }
    else if (json.error) {
        Logger_1.default.error(`Error in retrieved article [${articleId}]:`, json.error);
        return '';
    }
    return json; // This is HTML probably (the problem is that this is hard to know at this stage, buggy architecture)
};
const renderMCSArticle = (json, dump, articleId, articleDetail) => {
    let html = '';
    // set the first section (open by default)
    html += Templates_1.leadSectionTemplate({
        lead_display_title: json.lead.displaytitle,
        lead_section_text: json.lead.sections[0].text,
        strings: dump.strings,
    });
    // set all other section (closed by default)
    if (!dump.nodet && json.remaining.sections.length > 0) {
        const firstTocLevel = json.remaining.sections[0].toclevel;
        json.remaining.sections
            .forEach((oneSection, i) => {
            if (oneSection.toclevel === firstTocLevel) {
                html = html.replace(`__SUB_LEVEL_SECTION_${i}__`, ''); // remove unused anchor for subsection
                html += Templates_1.sectionTemplate({
                    section_index: i + 1,
                    section_id: oneSection.id,
                    section_anchor: oneSection.anchor,
                    section_line: oneSection.line,
                    section_text: oneSection.text,
                    strings: dump.strings,
                });
            }
            else {
                html = html.replace(`__SUB_LEVEL_SECTION_${i}__`, Templates_1.subSectionTemplate({
                    section_index: i + 1,
                    section_toclevel: oneSection.toclevel + 1,
                    section_id: oneSection.id,
                    section_anchor: oneSection.anchor,
                    section_line: oneSection.line,
                    section_text: oneSection.text,
                    strings: dump.strings,
                }));
            }
        });
    }
    const articleResourceNamespace = 'A';
    const categoryResourceNamespace = 'U';
    const slashesInUrl = articleId.split('/').length - 1;
    const upStr = '../'.repeat(slashesInUrl + 1);
    if (articleDetail.subCategories && articleDetail.subCategories.length) {
        const subCategories = articleDetail.subCategories.map((category) => {
            return {
                name: category.title.split(':').slice(1).join(':'),
                url: `${upStr}${categoryResourceNamespace}/${category.title}`,
            };
        });
        const groups = groupAlphabetical(subCategories);
        html += Templates_1.subCategoriesTemplate({
            strings: dump.strings,
            groups,
            prevArticleUrl: articleDetail.prevArticleId ? `${upStr}${categoryResourceNamespace}/${articleDetail.prevArticleId}` : null,
            nextArticleUrl: articleDetail.nextArticleId ? `${upStr}${categoryResourceNamespace}/${articleDetail.nextArticleId}` : null,
        });
    }
    if (articleDetail.pages && articleDetail.pages.length) {
        const pages = articleDetail.pages.map((page) => {
            return {
                name: page.title,
                url: `${upStr}${articleResourceNamespace}/${page.title}`,
            };
        });
        const groups = groupAlphabetical(pages);
        html += Templates_1.subPagesTemplate({
            strings: dump.strings,
            groups,
        });
    }
    if (articleDetail.categories && articleDetail.categories.length) {
        const categories = articleDetail.categories.map((category) => {
            return {
                name: category.title.split(':').slice(1).join(':'),
                url: `${upStr}${categoryResourceNamespace}/${category.title}`,
            };
        });
        html += Templates_1.categoriesTemplate({
            strings: dump.strings,
            categories,
        });
    }
    html = html.replace(`__SUB_LEVEL_SECTION_${json.remaining.sections.length}__`, ''); // remove the last subcestion anchor (all other anchor are removed in the forEach)
    return html;
};
const groupAlphabetical = (items) => {
    const groupsAlphabetical = items.reduce((acc, item) => {
        const groupId = item.name[0].toLocaleUpperCase();
        acc[groupId] = (acc[groupId] || []).concat(item);
        return acc;
    }, {});
    return Object.keys(groupsAlphabetical)
        .sort()
        .map((letter) => {
        return {
            title: letter,
            items: groupsAlphabetical[letter],
        };
    });
};
//# sourceMappingURL=articleRenderers.js.map