"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.simplifyGraph = exports.trimUnmirroredPages = exports.getCategoriesForArticles = void 0;
const RedisKvs_1 = require("./RedisKvs");
const Logger_1 = __importDefault(require("../Logger"));
const stores_1 = require("../stores");
const mw_api_1 = require("./mw-api");
const _1 = require(".");
function getCategoriesForArticles(articleStore, downloader, redis, deleteArticleStore = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const nextCategoriesBatch = new RedisKvs_1.RedisKvs(redis.client, `${Date.now()}-request`);
        Logger_1.default.log(`Fetching categories for [${yield articleStore.len()}] articles`);
        yield articleStore
            .iterateItems(downloader.speed, (articleKeyValuePairs, workerId) => __awaiter(this, void 0, void 0, function* () {
            const articleKeys = Object.keys(articleKeyValuePairs);
            Logger_1.default.log(`Worker [${workerId}] getting categories for articles ${Logger_1.default.logifyArray(articleKeys)}`);
            const pagesXCategoryId = Object.entries(articleKeyValuePairs)
                .reduce((acc, [aId, detail]) => {
                for (const cat of detail.categories || []) {
                    const catId = cat.title;
                    acc[catId] = (acc[catId] || []).concat({ title: detail.title, ns: detail.ns });
                }
                return acc;
            }, {});
            const foundCategoryIds = Object.keys(pagesXCategoryId);
            if (foundCategoryIds.length) {
                const existingArticles = yield stores_1.articleDetailXId.getMany(foundCategoryIds);
                const categoriesToGet = Object.entries(existingArticles).filter(([id, detail]) => !detail).map(([id]) => id);
                if (categoriesToGet.length) {
                    yield mw_api_1.getArticlesByIds(categoriesToGet, downloader, false);
                }
                const catDetails = yield stores_1.articleDetailXId.getMany(foundCategoryIds);
                for (const [id, detail] of Object.entries(catDetails)) {
                    if (!detail) {
                        continue;
                    }
                    const parentCategories = (detail.categories || [])
                        .reduce((acc, info) => {
                        const articleId = info.title;
                        return Object.assign(Object.assign({}, acc), { [articleId]: info });
                    }, {});
                    yield nextCategoriesBatch.setMany(parentCategories);
                    detail.pages = (detail.pages || []).concat(pagesXCategoryId[id]);
                    yield stores_1.articleDetailXId.set(id, detail);
                }
            }
        }));
        if (deleteArticleStore) {
            yield articleStore.flush();
        }
        const nextBatchSize = yield nextCategoriesBatch.len();
        if (nextBatchSize) {
            return getCategoriesForArticles(nextCategoriesBatch, downloader, redis, true);
        }
        else {
            return null;
        }
    });
}
exports.getCategoriesForArticles = getCategoriesForArticles;
function trimUnmirroredPages(downloader) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.log(`Trimming un-mirrored articles for [${yield stores_1.articleDetailXId.len()}] articles`);
        const numKeys = yield stores_1.articleDetailXId.len();
        let prevPercentProgress = -1;
        let processedArticles = 0;
        let modifiedArticles = 0;
        yield stores_1.articleDetailXId
            .iterateItems(downloader.speed, (articleKeyValuePairs, workerId) => __awaiter(this, void 0, void 0, function* () {
            for (const [articleId, articleDetail] of Object.entries(articleKeyValuePairs)) {
                processedArticles += 1;
                if (typeof articleDetail.missing === 'string') {
                    yield stores_1.articleDetailXId.delete(articleId);
                    modifiedArticles += 1;
                    // TODO: remove references to current article on delete
                    continue;
                }
                const categoriesXId = (articleDetail.categories || []).reduce((acc, c) => {
                    acc[c.title] = c;
                    return acc;
                }, {});
                const categoryIds = Object.keys(categoriesXId);
                const subCategoriesXId = (articleDetail.subCategories || []).reduce((acc, c) => {
                    acc[c.title] = c;
                    return acc;
                }, {});
                const subCategoryIds = Object.keys(subCategoriesXId);
                const pagesXId = (articleDetail.pages || []).reduce((acc, c) => {
                    acc[c.title] = c;
                    return acc;
                }, {});
                const pageIds = Object.keys(pagesXId);
                const [categoriesExist, subCategoriesExist, pagesExist,] = yield Promise.all([
                    categoryIds.length ? stores_1.articleDetailXId.exists(categoryIds) : Promise.resolve({}),
                    subCategoryIds.length ? stores_1.articleDetailXId.exists(subCategoryIds) : Promise.resolve({}),
                    pageIds.length ? stores_1.articleDetailXId.exists(pageIds) : Promise.resolve({}),
                ]);
                const existingCategories = Object.keys(categoriesExist).filter((key) => !!categoriesExist[key]);
                const existingSubCategories = Object.keys(subCategoriesExist).filter((key) => !!subCategoriesExist[key]);
                const existingPages = Object.keys(pagesExist).filter((key) => !!pagesExist[key]);
                let hasUpdated = false;
                const newCategoryKeys = _1.deDup(existingCategories || [], (p) => p);
                const newCategories = newCategoryKeys.map((key) => categoriesXId[key]);
                if (newCategories.length !== categoryIds.length) {
                    articleDetail.categories = newCategories;
                    hasUpdated = true;
                }
                const newSubCategoryKeys = _1.deDup(existingSubCategories || [], (p) => p);
                const newSubCategories = newSubCategoryKeys.map((key) => subCategoriesXId[key]);
                if (newSubCategories.length !== subCategoryIds.length) {
                    articleDetail.subCategories = newSubCategories;
                    hasUpdated = true;
                }
                const newPageKeys = _1.deDup(existingPages || [], (p) => p);
                const newPages = newPageKeys.map((key) => pagesXId[key]);
                if (newPages.length !== pageIds.length) {
                    articleDetail.pages = newPages;
                    hasUpdated = true;
                }
                if (hasUpdated) {
                    yield stores_1.articleDetailXId.set(articleId, articleDetail);
                    modifiedArticles += 1;
                }
                if (processedArticles % 100 === 0) {
                    const percentProgress = Math.floor(processedArticles / numKeys * 1000) / 10;
                    if (percentProgress !== prevPercentProgress) {
                        prevPercentProgress = percentProgress;
                        Logger_1.default.log(`Progress trimming un-mirrored articles [${processedArticles}/${numKeys}] [${percentProgress}%]`);
                    }
                }
            }
        }));
        return modifiedArticles;
    });
}
exports.trimUnmirroredPages = trimUnmirroredPages;
function simplifyGraph(downloader) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.log(`Simplifying graph (removing empty categories)`);
        const numKeys = yield stores_1.articleDetailXId.len();
        let prevPercentProgress = -1;
        let processedArticles = 0;
        let deletedNodes = 0;
        yield stores_1.articleDetailXId
            .iterateItems(downloader.speed, (articleKeyValuePairs, workerId) => __awaiter(this, void 0, void 0, function* () {
            for (const [articleId, articleDetail] of Object.entries(articleKeyValuePairs)) {
                processedArticles += 1;
                if (articleDetail.ns !== 14) {
                    continue; // Only trim category articles
                }
                const subArticles = (articleDetail.subCategories || []).concat(articleDetail.pages || []);
                const shouldRemoveNode = subArticles.length <= 3;
                if (shouldRemoveNode) {
                    // Update sub pages
                    // Add parent categories to child pages
                    const hasPages = articleDetail.pages && articleDetail.pages.length;
                    const scrapedPages = hasPages ? yield stores_1.articleDetailXId.getMany(articleDetail.pages.map((p) => p.title)) : {};
                    for (const [pageId, pageDetail] of Object.entries(scrapedPages)) {
                        if (pageDetail) {
                            pageDetail.categories = (pageDetail.categories || [])
                                .filter((c) => c && c.title !== articleDetail.title) // remove self
                                .concat(articleDetail.categories || []); // add parent categories
                            pageDetail.categories = _1.deDup(pageDetail.categories, (o) => o.title);
                            yield stores_1.articleDetailXId.set(pageId, pageDetail);
                        }
                    }
                    // Update parent categories
                    // Add children to parent categories
                    const hasCategories = articleDetail.categories && articleDetail.categories.length;
                    const scrapedCategories = hasCategories ? yield stores_1.articleDetailXId.getMany(articleDetail.categories.map((p) => p.title)) : {};
                    for (const [catId, catDetail] of Object.entries(scrapedCategories)) {
                        if (catDetail) {
                            const categoryDetail = Object.assign({ pages: [], subCategories: [] }, catDetail || {});
                            categoryDetail.pages = categoryDetail.pages.concat(articleDetail.pages);
                            categoryDetail.subCategories = categoryDetail.subCategories.concat(articleDetail.subCategories).filter((c) => c.title === articleDetail.title);
                            categoryDetail.pages = _1.deDup(categoryDetail.pages, (o) => o.title);
                            categoryDetail.subCategories = _1.deDup(categoryDetail.subCategories, (o) => o.title);
                            yield stores_1.articleDetailXId.set(catId, categoryDetail);
                        }
                    }
                    yield stores_1.articleDetailXId.delete(articleId);
                    deletedNodes += 1;
                }
            }
            if (processedArticles % 10 === 0) {
                const percentProgress = Math.floor(processedArticles / numKeys * 1000) / 10;
                if (percentProgress !== prevPercentProgress) {
                    prevPercentProgress = percentProgress;
                    Logger_1.default.log(`Progress simplifying graph [${processedArticles}/${numKeys}] [${percentProgress}%] deleted [${deletedNodes}]`);
                }
            }
        }));
        return { deletedNodes };
    });
}
exports.simplifyGraph = simplifyGraph;
//# sourceMappingURL=categories.js.map