"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mwRetToArticleDetail = exports.normalizeMwResponse = exports.getArticlesByNS = exports.getArticlesByIds = void 0;
const p_map_1 = __importDefault(require("p-map"));
const deepmerge = require("deepmerge");
const Logger_1 = __importDefault(require("../Logger"));
const stores_1 = require("../stores");
function getArticlesByIds(articleIds, downloader, log = true) {
    return __awaiter(this, void 0, void 0, function* () {
        let from = 0;
        let numThumbnails = 0;
        const MAX_BATCH_SIZE = 50;
        const MAX_URL_SIZE = 7900; // in bytes, approx.
        // using async iterator to spawn workers
        yield p_map_1.default(','.repeat(downloader.speed).split(',').map((_, i) => i), (workerId) => __awaiter(this, void 0, void 0, function* () {
            while (from < articleIds.length) {
                // Secure the request has the max articleIds as possible (within boudaries)
                const articleIdsBatch = articleIds.slice(from, from + MAX_BATCH_SIZE);
                let urlSize = encodeURIComponent(articleIdsBatch.join('|')).length;
                while (urlSize > MAX_URL_SIZE) {
                    urlSize -= encodeURIComponent(articleIdsBatch.pop()).length + 1;
                }
                // Udpate articleIds slicing boundaries
                const to = from + articleIdsBatch.length;
                if (log) {
                    const progressPercent = Math.floor(to / articleIds.length * 100);
                    Logger_1.default.log(`Worker [${workerId}] getting article range [${from}-${to}] of [${articleIds.length}] [${progressPercent}%]`);
                }
                from = to;
                // Nothing to do
                if (!articleIdsBatch.length) {
                    continue;
                }
                // Retrieve the details and save them in Redis
                const _articleDetails = yield downloader.getArticleDetailsIds(articleIdsBatch, numThumbnails < 100);
                const articlesWithThumbnail = Object.values(_articleDetails).filter((a) => !!a.thumbnail);
                numThumbnails += articlesWithThumbnail.length;
                const articleDetails = mwRetToArticleDetail(_articleDetails);
                for (const [articleId, articleDetail] of Object.entries(_articleDetails)) {
                    if (articleDetail.redirects && articleDetail.redirects.length) {
                        yield stores_1.redirectsXId.setMany(articleDetail.redirects.reduce((acc, redirect) => {
                            const rId = redirect.title;
                            return Object.assign(Object.assign({}, acc), { [rId]: { targetId: articleId, title: redirect.title } });
                        }, {}));
                    }
                }
                const existingArticleDetails = yield stores_1.articleDetailXId.getMany(Object.keys(articleDetails));
                yield stores_1.articleDetailXId.setMany(deepmerge(existingArticleDetails, articleDetails));
            }
        }), { concurrency: downloader.speed });
    });
}
exports.getArticlesByIds = getArticlesByIds;
function getArticlesByNS(ns, downloader, continueLimit) {
    return __awaiter(this, void 0, void 0, function* () {
        let totalArticles = 0;
        let chunk;
        do {
            chunk = yield downloader.getArticleDetailsNS(ns, chunk && chunk.gapContinue);
            yield stores_1.articleDetailXId.setMany(mwRetToArticleDetail(chunk.articleDetails));
            for (const [articleId, articleDetail] of Object.entries(chunk.articleDetails)) {
                yield stores_1.redirectsXId.setMany((articleDetail.redirects || []).reduce((acc, redirect) => {
                    const rId = redirect.title;
                    return Object.assign(Object.assign({}, acc), { [rId]: { targetId: articleId, title: redirect.title } });
                }, {}));
            }
            const numDetails = Object.keys(chunk.articleDetails).length;
            Logger_1.default.log(`Got [${numDetails}] articles chunk from namespace [${ns}]`);
            totalArticles += numDetails;
            // Only for testing purposes
            if (--continueLimit < 0)
                break;
        } while (chunk.gapContinue);
        Logger_1.default.log(`A total of [${totalArticles}] articles has been found in namespace [${ns}]`);
    });
}
exports.getArticlesByNS = getArticlesByNS;
function normalizeMwResponse(response) {
    if (!response) {
        return {};
    }
    const { normalized: _normalized, pages } = response;
    const normalized = (_normalized || []).reduce((acc, { from, to }) => {
        acc[to] = from;
        return acc;
    }, {});
    return Object.values(pages)
        .reduce((acc, page) => {
        const id = ((normalized.hasOwnProperty(page.title) && normalized[page.title]) || page.title || '');
        if (typeof id !== 'string' || !id) {
            Logger_1.default.warn(`Article Id is invalid - expected a string but got [${id}], converting to string and continuing`);
        }
        const articleId = String(id).replace(/ /g, '_');
        if (page.redirects) {
            page.redirects = page.redirects.map((redirect) => {
                // The API returns the redirect title (!?), we fake the
                // redirectId by putting the underscore. That way we
                // secure the URL rewritting works fine.
                redirect.title = String(redirect.title).replace(/ /g, '_');
                return redirect;
            });
        }
        if (articleId) {
            return Object.assign(Object.assign({}, acc), { [articleId]: page });
        }
        else {
            return acc;
        }
    }, {});
}
exports.normalizeMwResponse = normalizeMwResponse;
function mwRetToArticleDetail(obj) {
    const ret = {};
    for (const key of Object.keys(obj)) {
        const val = obj[key];
        const rev = val.revisions && val.revisions[0];
        const geo = val.coordinates && val.coordinates[0];
        let newThumbnail;
        if (val.thumbnail) {
            newThumbnail = {
                width: val.thumbnail.width,
                height: val.thumbnail.height,
                source: val.thumbnail.source,
            };
        }
        ret[key] = Object.assign(Object.assign(Object.assign({ title: val.title, categories: val.categories, subCategories: val.subCategories, thumbnail: newThumbnail, missing: val.missing }, (val.ns !== 0 ? { ns: val.ns } : {})), (rev ? { revisionId: rev.revid, timestamp: rev.timestamp } : {})), (geo ? { coordinates: `${geo.lat};${geo.lon}` } : {}));
    }
    return ret;
}
exports.mwRetToArticleDetail = mwRetToArticleDetail;
//# sourceMappingURL=mw-api.js.map